=== added file 'bin/ivle-fetchsubmissions'
--- bin/ivle-fetchsubmissions	1970-01-01 00:00:00 +0000
+++ bin/ivle-fetchsubmissions	2009-05-13 03:18:42 +0000
@@ -0,0 +1,328 @@
+#!/usr/bin/env python
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Program: Fetch Submissions
+# Author:  Matt Giuca
+
+# Script to retrieve all submissions for a particular project.
+# Requires root to run.
+
+from __future__ import with_statement
+
+import sys
+import os
+import shutil
+import datetime
+import codecs
+import optparse
+import zipfile
+import traceback
+
+import pysvn
+
+if os.getuid() != 0:
+    print >>sys.stderr, "Must run %s as root." % os.path.basename(sys.argv[0])
+    sys.exit()
+
+import ivle.config
+import ivle.database
+import ivle.util
+
+from ivle.database import Project, ProjectSet, Offering, Subject
+
+# Is Python version 2.6 or higher?
+PYTHON26 = map(int, sys.version[:3].split('.')) >= [2, 6]
+
+def fetch_submission(submission, target, svnclient, config, zip=False,
+    txt=False, verbose=False):
+    """Fetch a submission from a user's repository, and dump it in a given
+    directory.
+    @param submission: Submission object, detailing the submission.
+    @param target: Target directory for the project (will create a
+        subdirectory for each submission).
+    @param svnclient: pysvn.Client object.
+    @param config: Config object.
+    @param zip: If True, zips up the submission.
+    @param txt: If True, writes an extra text file with metadata about the
+        submission.
+    @param verbose: If True, writes the name of each submission to stdout.
+    """
+    # submission_name is the name of the user or group who owns the repo
+    submission_name = submission.assessed.principal.short_name
+    # target_final is the directory to place the files in
+    target_final = os.path.join(target,
+                                submission.assessed.principal.short_name)
+    target_final = target_final.encode('utf-8')
+    if os.path.exists(target_final):
+        # Remove the existing file or directory before re-checking out
+        if os.path.isdir(target_final):
+            ivle.util.safe_rmtree(target_final)
+        else:
+            os.remove(target_final)
+    url = get_repo_url(submission, config)
+    revision = pysvn.Revision(pysvn.opt_revision_kind.number,
+                              submission.revision)
+    svnclient.export(url, target_final, force=True,
+        revision=revision, recurse=True)
+
+    if txt:
+        # info_final is the directory to place the metadata info files in
+        info_final = target_final + ".txt"
+        write_submission_info(info_final, submission)
+
+    # If required, zip up the directory
+    if zip:
+        make_zip(target_final, target_final + ".zip")
+        # Remove the target tree
+        if os.path.isdir(target_final):
+            ivle.util.safe_rmtree(target_final)
+        else:
+            os.remove(target_final)
+
+    if verbose:
+        print "Exported submission by: %s (%s)" % (
+            submission.assessed.principal.short_name,
+            submission.assessed.principal.display_name)
+
+def get_repo_url(submission, config):
+    """Gets a local (file:) URL to the repository for a given submission.
+    This will consult submission.path to find the path within the repository
+    to check out.
+    @param submission: Submission object, detailing the submission.
+    @param config: Config object.
+    """
+    # NOTE: This code is mostly copied from services/usrmgt-server
+    if submission.assessed.is_group:
+        # The offering this group is in
+        offering = submission.assessed.project.project_set.offering
+        groupname = submission.assessed.principal.short_name
+        # The name of the repository directory within 'groups' is
+        # SUBJECT_YEAR_SEMESTER_GROUP
+        namespace = "_".join([offering.subject.short_name,
+            offering.semester.year, offering.semester.semester, groupname])
+        repo_path = os.path.join(config['paths']['svn']['repo_path'],
+                                'groups', namespace)
+    else:
+        # The name of the repository directory within 'users' is the username
+        username = submission.assessed.principal.short_name
+        repo_path = os.path.join(config['paths']['svn']['repo_path'],
+                                'users', username)
+
+    path_in_repo = submission.path
+    # Change an absolute path into a relative one (to the top of SVN)
+    if path_in_repo[:1] == os.sep or path_in_repo[:1] == os.altsep:
+        path_in_repo = path_in_repo[1:]
+
+    # Attach "file://" to the front of the absolute path, to make it a URL
+    return "file://" + os.path.join(os.path.abspath(repo_path), path_in_repo)
+
+def make_zip(source, dest):
+    """Zip up a directory tree or file. The ZIP file will always contain just
+    a single directory or file at the top level (it will not be a ZIP bomb).
+    XXX In Python 2.5 and earlier, this does NOT create empty directories
+    (it's not possible with the Python2.5 version of zipfile).
+    @param source: Path to a directory or file to zip up.
+    @param dest: Path to a zip file to create.
+    """
+    # NOTE: This code is mostly copied from ivle.zip (but very different)
+    zip = zipfile.ZipFile(dest, 'w')
+
+    # Write the source file/directory itself
+    # (If this is a directory it will NOT recurse)
+    if PYTHON26 or not os.path.isdir(source):
+        # Python < 2.6 errors if you add a directory
+        # (This means you can't add an empty directory)
+        zip.write(source, os.path.basename(source))
+
+    if os.path.isdir(source):
+        # All paths within the zip file are relative to relativeto
+        relativeto = os.path.dirname(source)
+        # Write the top-level directory
+        # Walk the directory tree
+        def error(err):
+            raise OSError("Could not access a file (zipping)")
+        for (dirpath, dirnames, filenames) in \
+            os.walk(source, onerror=error):
+            arc_dirpath = ivle.util.relpath(dirpath, relativeto)
+            # Python < 2.6 errors if you add a directory
+            # (This means you can't add an empty directory)
+            if PYTHON26:
+                filenames = dirnames + filenames
+            for filename in filenames:
+                zip.write(os.path.join(dirpath, filename),
+                            os.path.join(arc_dirpath, filename))
+
+    if not PYTHON26:
+        # XXX Write this _didModify attribute of zip, to trick it into writing
+        # footer bytes even if there are no files in the archive (otherwise it
+        # writes a 0-byte archive which is invalid).
+        # Note that in Python2.6 we avoid this by always writing the top-level
+        # file or directory, at least.
+        zip._didModify = True
+    zip.close()
+
+def write_submission_info(filename, submission):
+    """Write human-readable meta-data about a submission to a file.
+    @param filename: Filename to write to.
+    @param submission: Submission object.
+    """
+    with codecs.open(filename, 'w', 'utf-8') as f:
+        if submission.assessed.is_group:
+            # A group project
+            print >>f, "Group: %s (%s)" % (
+                submission.assessed.principal.short_name,
+                submission.assessed.principal.display_name)
+        else:
+            # A solo project
+            # Only show the two fields if they are different (only in rare
+            # circumstances)
+            if submission.assessed.principal != submission.submitter:
+                print >>f, "Author: %s (%s)" % (
+                    submission.assessed.principal.short_name,
+                    submission.assessed.principal.display_name)
+        print >>f, "Submitter: %s (%s)" % (
+            submission.submitter.short_name,
+            submission.submitter.display_name)
+        print >>f, "Date: %s" % (
+            submission.date_submitted.strftime("%Y-%m-%d %H:%M:%S"))
+        print >>f, "SVN Revision: %s" % submission.revision
+        print >>f, "SVN Path: %s" % submission.path
+
+def main(argv=None):
+    global store
+    if argv is None:
+        argv = sys.argv
+
+    usage = """usage: %prog [options] subject projname
+    (requires root)
+    Retrieves all submissions for a given project. Places each submission in
+    its own directory, in a subdirectory of '.'. Any errors are reported to
+    stderr (otherwise is silent).
+    subject/projname is the subject/project's short name.
+    """
+
+    # Parse arguments
+    parser = optparse.OptionParser(usage)
+    parser.add_option("-s", "--semester",
+        action="store", dest="semester", metavar="YEAR/SEMESTER",
+        help="Semester of the subject's offering (eg. 2009/1). "
+             "Defaults to the currently active semester.",
+        default=None)
+    parser.add_option("-d", "--dest",
+        action="store", dest="dest", metavar="PATH",
+        help="Destination directory (default to '.', creates a subdirectory, "
+            "so will not pollute PATH).",
+        default=".")
+    parser.add_option("-z", "--zip",
+        action="store_true", dest="zip",
+        help="Store each submission in a Zip file.",
+        default=False)
+    parser.add_option("-v", "--verbose",
+        action="store_true", dest="verbose",
+        help="Print out the name of each submission as it is extracted.",
+        default=False)
+    parser.add_option("--no-txt",
+        action="store_false", dest="txt",
+        help="Disable writing a text file with data about each submission.",
+        default=True)
+    (options, args) = parser.parse_args(argv[1:])
+
+    if len(args) < 2:
+        parser.print_help()
+        parser.exit()
+
+    subject_name = unicode(args[0])
+    project_name = unicode(args[1])
+
+    if options.semester is None:
+        year, semester = None, None
+    else:
+        try:
+            year, semester = options.semester.split('/')
+            if len(year) == 0 or len(semester) == 0:
+                raise ValueError()
+        except ValueError:
+            parser.error('Invalid semester (must have form "year/semester")')
+
+    svnclient = pysvn.Client()
+    config = ivle.config.Config(plugins=False)
+    store = ivle.database.get_store(config)
+
+    # Get the subject from the DB
+    subject = store.find(Subject,
+                     Subject.short_name == subject_name).one()
+    if subject is None:
+        print >>sys.stderr, "No subject with short name '%s'" % subject_name
+        return 1
+
+    # Get the offering from the DB
+    if semester is None:
+        # None specified - get the current offering from the DB
+        offerings = list(subject.active_offerings())
+        if len(offerings) == 0:
+            print >>sys.stderr, ("No active offering for subject '%s'"
+                                 % subject_name)
+            return 1
+        elif len(offerings) > 1:
+            print >>sys.stderr, ("Multiple active offerings for subject '%s':"
+                                 % subject_name)
+            print >>sys.stderr, "Please use one of:"
+            for offering in offerings:
+                print >>sys.stderr, ("    --semester=%s/%s"
+                    % (offering.semester.year, offering.semester.semester))
+            return 1
+        else:
+            offering = offerings[0]
+    else:
+        # Get the offering for the specified semester
+        offering = subject.offering_for_semester(year, semester)
+        if offering is None:
+            print >>sys.stderr, (
+                "No offering for subject '%s' in semester %s/%s"
+                % (subject_name, year, semester))
+            return 1
+
+    # Get the project from the DB
+    project = store.find(Project,
+                         Project.project_set_id == ProjectSet.id,
+                         ProjectSet.offering == offering,
+                         Project.short_name == project_name).one()
+    if project is None:
+        print >>sys.stderr, "No project with short name '%s'" % project_name
+        return 1
+
+    # Target directory is DEST/subject/year/semester/project
+    target_dir = os.path.join(options.dest, subject_name,
+        offering.semester.year, offering.semester.semester, project_name)
+    if not os.path.exists(target_dir):
+        os.makedirs(target_dir)
+
+    for submission in project.latest_submissions:
+        try:
+            fetch_submission(submission, target_dir, svnclient, config,
+                             zip=options.zip, txt=options.txt,
+                             verbose=options.verbose)
+        except Exception, e:
+            # Catch all exceptions (to ensure if one student has a problem, it
+            # is reported, and we can continue)
+            print >>sys.stderr, "ERROR on submission for %s:" % (
+                submission.assessed.principal.display_name)
+            traceback.print_exc()
+
+if __name__ == "__main__":
+    sys.exit(main(sys.argv))

=== modified file 'bin/ivle-marks'
--- bin/ivle-marks	2009-04-28 05:06:00 +0000
+++ bin/ivle-marks	2009-05-11 04:42:18 +0000
@@ -27,7 +27,6 @@
 import csv
 import datetime
 import optparse
-from xml.dom import minidom
 
 if os.getuid() != 0:
     print >>sys.stderr, "Must run %s as root." % os.path.basename(sys.argv[0])
@@ -145,7 +144,7 @@
     else:
         cutoff = None
 
-    store = ivle.database.get_store(ivle.config.Config())
+    store = ivle.database.get_store(ivle.config.Config(plugins=False))
 
     # Get the subject from the DB
     subject = store.find(ivle.database.Subject,

=== added file 'bin/ivle-refreshfilesystem'
--- bin/ivle-refreshfilesystem	1970-01-01 00:00:00 +0000
+++ bin/ivle-refreshfilesystem	2009-05-11 04:51:08 +0000
@@ -0,0 +1,38 @@
+#!/usr/bin/env python
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+"""Refresh parts of the filesystem that are generated from the database.
+
+In particular, the Subversion authorisation files are rewritten.
+"""
+
+import sys
+
+from ivle.config import Config
+from ivle.database import get_store
+import ivle.makeuser
+
+config = Config()
+store = get_store(config)
+
+print >>sys.stderr, "Rebuilding Subversion user configuration..."
+ivle.makeuser.rebuild_svn_config(store, config)
+print >>sys.stderr, "Rebuilding Subversion group configuration..."
+ivle.makeuser.rebuild_svn_group_config(store, config)
+
+

=== modified file 'ivle/database.py'
--- ivle/database.py	2009-05-04 01:52:04 +0000
+++ ivle/database.py	2009-05-12 13:50:49 +0000
@@ -28,6 +28,7 @@
 
 from storm.locals import create_database, Store, Int, Unicode, DateTime, \
                          Reference, ReferenceSet, Bool, Storm, Desc
+from storm.expr import Select, Max
 from storm.exceptions import NotOneError, IntegrityError
 
 from ivle.worksheet.rst import rst
@@ -116,9 +117,15 @@
 
     @property
     def display_name(self):
+        """Returns the "nice name" of the user or group."""
         return self.fullname
 
     @property
+    def short_name(self):
+        """Returns the database "identifier" name of the user or group."""
+        return self.login
+
+    @property
     def password_expired(self):
         fieldval = self.pass_exp
         return fieldval is not None and datetime.datetime.now() > fieldval
@@ -379,6 +386,17 @@
 
         return enrolment
 
+    def get_members_by_role(self, role):
+        return Store.of(self).find(User,
+                Enrolment.user_id == User.id,
+                Enrolment.offering_id == self.id,
+                Enrolment.role == role
+                )
+
+    @property
+    def students(self):
+        return self.get_members_by_role(u'student')
+
 class Enrolment(Storm):
     """An enrolment of a user in an offering.
 
@@ -438,6 +456,18 @@
     def get_permissions(self, user):
         return self.offering.get_permissions(user)
 
+    @property
+    def assigned(self):
+        """Get the entities (groups or users) assigned to submit this project.
+
+        This will be a Storm ResultSet.
+        """
+        #If its a solo project, return everyone in offering
+        if self.max_students_per_group is None:
+            return self.offering.students
+        else:
+            return self.project_groups
+
 class Project(Storm):
     """A student project for which submissions can be made."""
 
@@ -494,6 +524,19 @@
     def get_permissions(self, user):
         return self.project_set.offering.get_permissions(user)
 
+    @property
+    def latest_submissions(self):
+        """Return the latest submission for each Assessed."""
+        return Store.of(self).find(ProjectSubmission,
+            Assessed.project_id == self.id,
+            ProjectSubmission.assessed_id == Assessed.id,
+            ProjectSubmission.date_submitted == Select(
+                    Max(ProjectSubmission.date_submitted),
+                    ProjectSubmission.assessed_id == Assessed.id,
+                    tables=ProjectSubmission
+            )
+        )
+
 
 class ProjectGroup(Storm):
     """A group of students working together on a project."""
@@ -522,7 +565,13 @@
 
     @property
     def display_name(self):
-        return '%s (%s)' % (self.nick, self.name)
+        """Returns the "nice name" of the user or group."""
+        return self.nick
+
+    @property
+    def short_name(self):
+        """Returns the database "identifier" name of the user or group."""
+        return self.name
 
     def get_projects(self, offering=None, active_only=True):
         '''Find projects that the group can submit.
@@ -593,6 +642,11 @@
             self.user or self.project_group, self.project)
 
     @property
+    def is_group(self):
+        """True if the Assessed is a group, False if it is a user."""
+        return self.project_group is not None
+
+    @property
     def principal(self):
         return self.project_group or self.user
 

=== modified file 'ivle/makeuser.py'
--- ivle/makeuser.py	2009-05-05 06:46:58 +0000
+++ ivle/makeuser.py	2009-05-11 04:28:07 +0000
@@ -31,7 +31,7 @@
 
 import ivle.config
 from ivle.database import (User, ProjectGroup, Assessed, ProjectSubmission,
-        Project, ProjectSet, Offering, Enrolment)
+        Project, ProjectSet, Offering, Enrolment, Subject, Semester)
 
 def chown_to_webserver(filename):
     """chown a directory and its contents to the web server.
@@ -136,6 +136,7 @@
 
 """ % {'time': time.asctime()})
 
+    group_members_cache = {}
     for group in store.find(ProjectGroup):
         offering = group.project_set.offering
         reponame = "_".join([offering.subject.short_name,
@@ -144,10 +145,60 @@
                              group.name])
 
         f.write("[%s:/]\n" % reponame)
+        if group.id not in group_members_cache:
+            group_members_cache[group.id] = set()
         for user in group.members:
+            group_members_cache[group.id].add(user.login)
             f.write("%s = rw\n" % user.login)
         f.write("\n")
 
+    # Now we need to grant offering tutors and lecturers access to the latest
+    # submissions in their offerings. There are much prettier ways to do this,
+    # but a lot of browser requests call this function, so it needs to be
+    # fast. We can grab all of the paths needing authorisation directives with
+    # a single query, and we cache the list of viewers for each offering.
+    offering_viewers_cache = {}
+    for (ssn, year, sem, name, psid, pspath, gid, offeringid) in store.find(
+        (Subject.short_name, Semester.year, Semester.semester,
+         ProjectGroup.name, ProjectSubmission.id, ProjectSubmission.path,
+         ProjectGroup.id, Offering.id),
+            Assessed.id == ProjectSubmission.assessed_id,
+            ProjectGroup.id == Assessed.project_group_id,
+            Project.id == Assessed.project_id,
+            ProjectSet.id == Project.project_set_id,
+            Offering.id == ProjectSet.offering_id,
+            Subject.id == Offering.subject_id,
+            Semester.id == Offering.semester_id,
+            ProjectSubmission.date_submitted == Select(
+                    Max(ProjectSubmission.date_submitted),
+                    ProjectSubmission.assessed_id == Assessed.id,
+                    tables=ProjectSubmission
+            )
+        ):
+
+        reponame = "_".join([ssn, year, sem, name])
+
+        # Do we already have the list of logins authorised for this offering
+        # cached? If not, get it.
+        if offeringid not in offering_viewers_cache:
+            offering_viewers_cache[offeringid] = list(store.find(
+                    User.login,
+                    User.id == Enrolment.user_id,
+                    Enrolment.offering_id == offeringid,
+                    Enrolment.role.is_in((u'tutor', u'lecturer'))
+                )
+            )
+
+        f.write("""
+# Submission %(id)d
+[%(repo)s:%(path)s]
+""" % {'repo': reponame, 'id': psid, 'path': pspath})
+
+        for viewer_login in offering_viewers_cache[offeringid]:
+            # Skip existing group members, or they can't write to it any more.
+            if viewer_login not in group_members_cache[gid]:
+                f.write("%s = r\n" % viewer_login)
+
     f.close()
     os.rename(temp_name, conf_name)
     chown_to_webserver(conf_name)

=== modified file 'ivle/util.py'
--- ivle/util.py	2009-04-28 06:55:03 +0000
+++ ivle/util.py	2009-05-12 15:21:57 +0000
@@ -22,6 +22,8 @@
 # Contains common utility functions.
 
 import os
+import sys
+import stat
 
 class IVLEError(Exception):
     """Legacy general IVLE exception.
@@ -94,6 +96,25 @@
     else:
         return tuple(splitpath)
 
+def relpath(path, start=os.path.curdir):
+    """Return a relative version of a path.
+    XXX Backported from Python 2.6 posixpath.py.
+    """
+
+    if not path:
+        raise ValueError("no path specified")
+
+    start_list = os.path.abspath(start).split(os.path.sep)
+    path_list = os.path.abspath(path).split(os.path.sep)
+
+    # Work out how much of the filepath is shared by start and path.
+    i = len(os.path.commonprefix([start_list, path_list]))
+
+    rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
+    if not rel_list:
+        return os.path.curdir
+    return os.path.join(*rel_list)
+
 def incomplete_utf8_sequence(byteseq):
     """Calculate the completeness of a UTF-8 encoded string.
 
@@ -184,3 +205,55 @@
     """
     return dict((k, getattr(obj, k))
         for k in attrnames if not k.startswith('_'))
+
+def safe_rmtree(path, ignore_errors=False, onerror=None):
+    """Recursively delete a directory tree.
+
+    Copied from shutil.rmtree from Python 2.6, which does not follow symbolic
+    links (it is otherwise unsafe to call as root on untrusted directories; do
+    not use shutil.rmtree in this case, as you may be running Python 2.5).
+
+    If ignore_errors is set, errors are ignored; otherwise, if onerror
+    is set, it is called to handle the error with arguments (func,
+    path, exc_info) where func is os.listdir, os.remove, or os.rmdir;
+    path is the argument to that function that caused it to fail; and
+    exc_info is a tuple returned by sys.exc_info().  If ignore_errors
+    is false and onerror is None, an exception is raised.
+
+    """
+    if ignore_errors:
+        def onerror(*args):
+            pass
+    elif onerror is None:
+        def onerror(*args):
+            raise
+    try:
+        if os.path.islink(path):
+            # symlinks to directories are forbidden, see bug #1669
+            raise OSError("Cannot call safe_rmtree on a symbolic link")
+    except OSError:
+        onerror(os.path.islink, path, sys.exc_info())
+        # can't continue even if onerror hook returns
+        return
+    names = []
+    try:
+        names = os.listdir(path)
+    except os.error, err:
+        onerror(os.listdir, path, sys.exc_info())
+    for name in names:
+        fullname = os.path.join(path, name)
+        try:
+            mode = os.lstat(fullname).st_mode
+        except os.error:
+            mode = 0
+        if stat.S_ISDIR(mode):
+            safe_rmtree(fullname, ignore_errors, onerror)
+        else:
+            try:
+                os.remove(fullname)
+            except os.error, err:
+                onerror(os.remove, fullname, sys.exc_info())
+    try:
+        os.rmdir(path)
+    except os.error:
+        onerror(os.rmdir, path, sys.exc_info())

=== modified file 'ivle/webapp/admin/offeringservice.py'
--- ivle/webapp/admin/offeringservice.py	2009-05-04 03:48:10 +0000
+++ ivle/webapp/admin/offeringservice.py	2009-05-06 00:40:43 +0000
@@ -37,7 +37,10 @@
     def add_projectset(self, req, group_size):
         """Add a new ProjectSet"""
         new_projectset = ProjectSet()
-        new_projectset.max_students_per_group = int(group_size)
+        if group_size == '':
+            new_projectset.max_students_per_group = None
+        else:
+            new_projectset.max_students_per_group = int(group_size)
         new_projectset.offering = self.context
 
         req.store.add(new_projectset)

=== modified file 'ivle/webapp/admin/subject-media/project.css'
--- ivle/webapp/admin/subject-media/project.css	2009-05-04 05:15:23 +0000
+++ ivle/webapp/admin/subject-media/project.css	2009-05-12 14:59:03 +0000
@@ -54,3 +54,33 @@
 .project-title img {
   vertical-align: top;
 }
+
+.project_list {
+  text-align: left;
+  border-collapse: collapse;
+}
+
+.project_list td, .project_list th {
+  padding: 0.2em 0.7em;
+  border-bottom: 1px solid #ccc;
+}
+
+.project_list th {
+  border-bottom: 2px solid gray;
+}
+
+.project_list tbody td {
+  color: #444;
+}
+
+.project_list tbody tr:hover {
+  background-color: #efefef;
+}
+
+.project_list td:first-child, .project_list th:first-child {
+  padding-left: 0;
+}
+
+.project_list tbody tr:hover td {
+  color: #111;
+}

=== modified file 'ivle/webapp/admin/subject-media/project.js'
--- ivle/webapp/admin/subject-media/project.js	2009-05-04 04:21:08 +0000
+++ ivle/webapp/admin/subject-media/project.js	2009-05-12 11:49:31 +0000
@@ -1,7 +1,7 @@
 $(document).ready(function(){
     $("#new_projectset_form").submit(add_projectset);
     $(".new_project").submit(add_project);
-    $('li').slideDown();
+    $('li').show();
     $('.add-project-link').click(show_add);
 });
 

=== modified file 'ivle/webapp/admin/subject.py'
--- ivle/webapp/admin/subject.py	2009-05-04 03:48:10 +0000
+++ ivle/webapp/admin/subject.py	2009-05-12 13:15:09 +0000
@@ -23,7 +23,9 @@
 # A sample / testing application for IVLE.
 
 import os
+import os.path
 import urllib
+import urlparse
 import cgi
 
 from storm.locals import Desc
@@ -218,18 +220,32 @@
         if self.context is None:
             raise NotFound()
 
+    def build_subversion_url(self, svnroot, submission):
+        princ = submission.assessed.principal
+
+        if isinstance(princ, User):
+            path = 'users/%s' % princ.login
+        else:
+            path = 'groups/%s_%s_%s_%s' % (
+                    princ.project_set.offering.subject.short_name,
+                    princ.project_set.offering.semester.year,
+                    princ.project_set.offering.semester.semester,
+                    princ.name
+                    )
+        return urlparse.urljoin(
+                    svnroot,
+                    os.path.join(path, submission.path[1:] if
+                                       submission.path.startswith(os.sep) else
+                                       submission.path))
+
     def populate(self, req, ctx):
+        self.plugin_styles[Plugin] = ["project.css"]
+
         ctx['format_datetime_short'] = ivle.date.format_datetime_for_paragraph
-
+        ctx['build_subversion_url'] = self.build_subversion_url
+        ctx['svn_addr'] = req.config['urls']['svn_addr']
         ctx['project'] = self.context
-        ctx['assesseds'] = self.context.assesseds
-
-        ctx['submissions'] = []
-        for assessed in self.context.assesseds:
-            if assessed.submissions.count() > 0:
-                ctx['submissions'].append(
-                        assessed.submissions.order_by(ProjectSubmission.date_submitted)[-1])
-
+        ctx['user'] = req.user
 
 class Plugin(ViewPlugin, MediaPlugin):
     urls = [

=== modified file 'ivle/webapp/admin/templates/project.html'
--- ivle/webapp/admin/templates/project.html	2009-05-04 02:12:58 +0000
+++ ivle/webapp/admin/templates/project.html	2009-05-12 14:59:03 +0000
@@ -7,11 +7,43 @@
     <h1>${project.name} for ${project.project_set.offering.subject.name}</h1>
     <div id="ivle_padding">
       <p>${project.synopsis}</p>
-      <p>Submissions: ${len(submissions)}/${project.assesseds.count()}</p>
-      <h3 py:if="len(submissions) > 0">Latest Submissions</h3>
-      <py:for each="submission in submissions">
-        <p>Submitted ${format_datetime_short(submission.date_submitted)} on behalf of <em>${submission.assessed.principal.display_name}</em> by <em>${submission.submitter.display_name}</em></p>
-      </py:for>
+      <p>Submissions: ${project.latest_submissions.count()}/${project.project_set.assigned.count()}</p>
+      <table id="project_submissions" class="project_list" summary="Latest Submissions"
+             py:if="project.latest_submissions.count() > 0">
+        <thead>
+          <tr>
+            <th>Group</th>
+            <th>Submitter</th>
+            <th>Date</th>
+            <th>Download</th>
+          </tr>
+        </thead>
+        <tbody>
+        <py:for each="submission in project.latest_submissions">
+          <tr>
+          <py:choose test="submission.assessed.principal is submission.submitter">
+            <td py:when="True">-</td>
+            <td py:otherwise=""><span title="${submission.assessed.principal.display_name}">${submission.assessed.principal.short_name}</span></td>
+          </py:choose>
+            <td><span title="${submission.submitter.display_name}">${submission.submitter.short_name}</span></td>
+            <td>${submission.date_submitted.strftime("%Y-%m-%d %H:%M:%S")}</td>
+            <td>svn co --username ${user.login} -r${submission.revision} ${build_subversion_url(svn_addr, submission)}</td>
+          </tr>
+        </py:for>
+        </tbody>
+      </table>
+      <br />
+      <table id="project_assigneds" class="project_list">
+        <thead><tr><th>Assigned</th></tr></thead>
+        <tbody>
+        <py:for each="assignee in project.project_set.assigned">
+          <tr>
+            <td><span title="${assignee.display_name}">${assignee.short_name}</span></td>
+            <!-- <td><a>Grant an extension</a></td> -->
+          </tr>
+        </py:for>
+        </tbody>
+      </table>
     </div>
   </body>
 </html>

=== modified file 'ivle/webapp/admin/templates/project_fragment.html'
--- ivle/webapp/admin/templates/project_fragment.html	2009-05-04 05:03:58 +0000
+++ ivle/webapp/admin/templates/project_fragment.html	2009-05-12 11:49:31 +0000
@@ -3,7 +3,9 @@
     id='project_${project.id}'>
   <div class='project-title'>
     <a href="${project_url}"><strong>${project.name}</strong></a>
-    <a href='#'><img src='/+media/ivle.webapp.core/images/interface/cross.png' alt="Delete" title="Delete project" /></a>
+    <!--<a href='#'><img
+    src='/+media/ivle.webapp.core/images/interface/cross.png' alt="Delete"
+    title="Delete project" /></a> -->
   </div>
   <pre class='project-synopsis'>${project.synopsis}</pre>
 </li>

=== modified file 'ivle/webapp/admin/templates/subjects.html'
--- ivle/webapp/admin/templates/subjects.html	2009-05-03 07:08:10 +0000
+++ ivle/webapp/admin/templates/subjects.html	2009-05-13 02:41:25 +0000
@@ -19,7 +19,7 @@
               <a href="${offering_url(enrolment.offering)}/+worksheets">${enrolment.offering.subject.name}</a>
               <py:if test="'edit' in enrolment.offering.get_permissions(user)">
                 -
-                <a href="${offering_url(enrolment.offering)}/+groups">(administer groups)</a>
+                <a href="${offering_url(enrolment.offering)}/+projects">(administer projects)</a>
                 <a href="${offering_url(enrolment.offering)}/+enrolments/+new">(enrol users)</a>
               </py:if>
             </li>

=== modified file 'ivle/webapp/admin/user.py'
--- ivle/webapp/admin/user.py	2009-02-25 08:21:38 +0000
+++ ivle/webapp/admin/user.py	2009-05-11 04:21:11 +0000
@@ -55,7 +55,7 @@
         return user
 
 class UserSettingsView(XHTMLView):
-    template = 'user-settings.html'
+    template = 'templates/user-settings.html'
     tab = 'settings'
     permission = 'edit'
 

=== modified file 'ivle/webapp/submit/__init__.py'
--- ivle/webapp/submit/__init__.py	2009-05-05 06:37:51 +0000
+++ ivle/webapp/submit/__init__.py	2009-05-12 08:29:47 +0000
@@ -31,6 +31,7 @@
 from ivle.webapp.base.plugins import ViewPlugin
 
 import ivle.date
+import ivle.chat
 
 
 class SubmitView(XHTMLView):
@@ -92,6 +93,33 @@
 
             project.submit(self.context, self.path, revision, req.user)
 
+            # The Subversion configuration needs to be updated, to grant
+            # tutors and lecturers access to this submission. We have to 
+            # commit early so usrmgt-server can see the new submission.
+            req.store.commit()
+
+            # Instruct usrmgt-server to rebuild the SVN group authz file.
+            msg = {'rebuild_svn_group_config': {}}
+            usrmgt = ivle.chat.chat(req.config['usrmgt']['host'],
+                                    req.config['usrmgt']['port'],
+                                    msg,
+                                    req.config['usrmgt']['magic'],
+                                    )
+
+            if usrmgt.get('response') in (None, 'failure'):
+                raise Exception("Failure creating repository: " + str(usrmgt))
+
+            # Instruct usrmgt-server to rebuild the SVN user authz file.
+            msg = {'rebuild_svn_config': {}}
+            usrmgt = ivle.chat.chat(req.config['usrmgt']['host'],
+                                    req.config['usrmgt']['port'],
+                                    msg,
+                                    req.config['usrmgt']['magic'],
+                                    )
+
+            if usrmgt.get('response') in (None, 'failure'):
+                raise Exception("Failure creating repository: " + str(usrmgt))
+
             self.template = 'submitted.html'
             ctx['project'] = project
 

=== modified file 'services/svnlogservice'
--- services/svnlogservice	2009-02-05 13:54:19 +0000
+++ services/svnlogservice	2009-05-13 03:15:26 +0000
@@ -51,12 +51,13 @@
         code = e[1][0][1]
         # See subversion/include/svn_error_codes.h.
         # 155007: WC_NOT_DIRECTORY.
+        # 160006: FS_NO_SUCH_REVISION
         # 160013: FS_NOT_FOUND
         # 200005: UNVERSIONED_RESOURCE
-        if code in (155007, 160013, 200005):
+        if code in (155007, 160006, 160013, 200005):
             error = 'notfound'
         else:
-            error = '%s (code %d)' % (error, code) 
+            error = '%s (code %d)' % (error, code)
     except IndexError:
         pass
 

=== modified file 'setup/util.py'
--- setup/util.py	2009-03-03 01:48:48 +0000
+++ setup/util.py	2009-05-12 08:44:07 +0000
@@ -312,6 +312,7 @@
     list_user_binaries = [
         "bin/ivle-enrol",
         "bin/ivle-enrolallusers",
+        "bin/ivle-fetchsubmissions",
         "bin/ivle-listusers",
         "bin/ivle-makeuser",
         "bin/ivle-marks",
@@ -323,4 +324,5 @@
         "bin/ivle-buildjail",
         "bin/ivle-addexercise",
         "bin/ivle-cloneworksheets",
+        "bin/ivle-refreshfilesystem",
     ]

